#!/bin/bash
#######################################################################
# Author : O.Romain.Jaillet-ramey (orj+bash+dnssec (at) lab3w.fr)
# Date Create : 20220926
# Date Modified : 20221005
# Desc : Script to check expiration and sign zone (last day) DNSSEC
#######################################################################

#####-------------------------- CONFIG ----------------------------#####

date_year="$(date +%Y)"
date_month="$(date +%m)"
date_day="$(date +%d)"
date_hour="$(date +%H:%M:%S)"
date="${date_year}-${date_month}-${date_day} ${date_hour}"
timestamp=$(date -d "${date}" +%s)

month_more_1m=$((60 * 60 * 24 * 30))

#####----#####

force=0

dnssec_keys="/var/cache/bind/"
dnssec_keys="/etc/bind/keys"
dnssec_keys_algo_8="/etc/bind/keys/"
dnssec_keys_algo_13="/etc/bind/keys"

#####-------------------------- /CONFIG ---------------------------#####

#####-------------------------- FUNCTIONS -------------------------#####

usage() {                                 # Function: Print a help message.
	echo "Usage: $0 [ -d domain.tld ] [ -f /etc/bind/master/domain.tld ] [ -a 8|13] [ --force ]" 1>&2 
}
exit_abnormal() {                         # Function: Exit with error.
	usage
	exit 1
}

script_desc() {

	echo "#####---------------------------------------------------------"
	echo "#"
	echo "# DESC : Script to check expiration and sign zone (last day) DNSSEC"
	echo "#"
	echo "# ACTUAL DATE : ${date}"
	#echo "# ACTUAL TIMESTAMP : ${timestamp}"
	#echo "# ACTUAL TIMESTAMP + 1 MONTH : ${timestamp} + ${month_more_1m} = $((${timestamp} + ${month_more_1m}))\n"

	expire_timestamp="$((${timestamp} + ${month_more_1m}))"
	expire_date=$(date -d @${expire_timestamp} '+%Y-%m-%d %H:%M:%S')

	#echo "# EXPIRE DATE : ${expire_date}"
	echo "#"
	echo -e "#####---------------------------------------------------------\n"

	echo -e "#####---------------------------------------------------------\n"

}
zone_infos() {

	resign=0

	zone=${zone}
	zoneFile=${zoneFile}
	zoneFileSigned="${zoneFile}.signed"

	date_zonefile="$(stat ${zoneFile} | egrep '^Modif.' | cut -d' ' -f2-3 | cut -d'.' -f1)"
	time_zonefile=$(date -d "${date_zonefile}" +%s)

	time_expire="$((${time_zonefile} + ${month_more_1m}))"
	date_expire=$(date -d @${time_expire} '+%Y-%m-%d %H:%M:%S')

	echo "+ DNS Zone : ${zone}"
	echo ' \'
	echo "  +- Zone file : ${zoneFile}"
	echo "  +- Zone file signed : ${zoneFileSigned}"

	echo "  +-"
	echo '  | \'
	echo "  |  +-> Date Create : ${date_zonefile}"
	echo "  |  +-> Date Expire : ${date_expire}"
	echo "  |"

	days_expire_time="$((${time_expire} - ${timestamp}))"
	days_expire="$((${days_expire_time} / 60 / 60 / 24))"
#	days_expire=1
	
	if [ ${days_expire} -le 1 ] || [ ${force} -eq 1 ]; then
		resign=1
		echo "  +-> This zone updating now"
	else
		echo "  +-> This zone OK : Expire in ${days_expire} days (no update)"
	fi
}

zone_resign() {

	if [ ${resign} -eq 1 ]; then
		echo '  \'
		echo "   +-> This zone expire in ${days_expire} days"	
		echo '   |'
		echo "   +- Re checkzone into zone file....."
		echo "   |"
		zonefile_check=$(named-checkzone $zone ${zoneFile} | egrep 'OK|failed|errors')

		if [ ${zonefile_check} = 'OK' ]; then

			zonefile_soa=$(named-checkzone $zone $zoneFile | egrep -ho '[0-9]{8,}')
#			zonefile_soa="2022092702"
                	zonefile_soa_date=$(echo ${zonefile_soa:0:8})
			zonefile_soa_serial="$(echo ${zonefile_soa:8})"

			echo "    +- Check syntax zone file - ${zonefile_check}"
			echo '    | \'
			echo "    |  +-> SOA Date : ${zonefile_soa_date}"
			echo "    |  +-> SOA Serial : ${zonefile_soa_serial}"
			echo "    |"
			echo "    +- Modify zone file...."
			echo '      \'

			new_date_year_soa="$(date +%Y)"
			new_date_month_soa="$(date +%m)"
			new_date_day_soa="$(date +%d)"
			new_serial_soa="01"

			new_date_soa="${new_date_year_soa}${new_date_month_soa}${new_date_day_soa}"

			echo "       +- Change SOA : "
			if [ ${zonefile_soa_date} -eq ${new_date_soa} ]; then
				new_soa="${zonefile_soa_date}$(printf '%02d' $((${zonefile_soa_serial#0*} + 1 )))"
			else
				new_soa=${new_date_soa}${new_serial_soa}
			fi
			
			echo '       | \'
			echo "       |  +-> ${zonefile_soa} -> ${new_soa}"
			echo "       |"
			
			#... modify zone file with sed -i
			sed -i 's/'${zonefile_soa}'/'${new_soa}'/' ${zoneFile}

			echo "       +-> Zone file modified - OK"
			
			dnssec-signzone -N INCREMENT -o ${zone} -K ${dnssec_keys} -t ${zoneFile}
			echo "       +-> Zone file signing - OK"

			chown bind:bind ${zoneFileSigned}
			echo "       +-> Zone file signed - Assignment of user:group \"bind\" rights on ${zoneFileSigned}"

			if [ ${force} -eq 1 ]; then
				/etc/init.d/bind9 restart
				echo "       +-> Bind server DNS reload"
			
				exit 0
			else
				exit 150
			fi
		else
			echo "   +-> This zone ${zone} in ERROR - Check zone file ${zoneFile}"
		fi
	fi

	#.. for test exit status
	#exit 151
}

#####-------------------------- /FUNCTIONS ------------------------#####

#####-------------------------- CHECK -----------------------------#####

ID=`id -u`

if [ "x$ID" != "x0" ]; then
	echo "Run as root!"
        exit 1
fi

while getopts "d:f:a:-:" options; do

        case "${options}" in

        d)
#                echo -n "Domain Name : "
                zone=${OPTARG}
#                echo ${zone}
        ;;
        f)
#                echo -n "Zone file : "
               zoneFile=${OPTARG}
#                echo ${zoneFile}

        ;;
	a)
		algorithm=${OPTARG}
		if [ ${algorithm} -eq 8 ]; then
			dnssec_keys=${dnssec_keys_algo_8}
		elif [ ${algorithm} -eq 13 ]; then
			dnssec_keys=${dnssec_keys_algo_13}
		else
			dnssec_keys=${dnssec_keys}
		fi
	;;
	-)
		case "${OPTARG}" in
			force)
				val="${!OPTIND}"; OPTIND=$(( $OPTIND + 1 ))
#				echo "Parsing option: '--${OPTARG}', value: '${val}'" >&2;
				force=1;				
			;;
			force=*)
				val=${OPTARG#*=}
				opt=${OPTARG%=$val}
#				echo "Parsing option: '--${opt}', value: '${val}'" >&2
				force=${val}
			;;
			*)
				if [ "$OPTERR" = 1 ] && [ "${options:0:1}" != ":" ]; then
					echo "Unknown option --${OPTARG}" >&2
				fi
			;;
		esac
	;;
        esac

done
shift

#####-------------------------- /CHECK ----------------------------#####

#####-------------------------- SCRIPT ----------------------------#####

if [ "${zone}" != "" ]; then

	if [ "${zoneFile}" != "" ]; then
		
#		script_desc
		zone_infos
		zone_resign
		
		echo -e "\n#####---------------------------------------------------------"

	else
		echo "Configure zone file: -f /etc/bind/masters/domain.tld.hosts"
		usage
	fi
else
	echo "Configure the zone name: -d domain.tld"
	usage
fi

#exit 0
#####-------------------------- /SCRIPT ----------------------------#####
